package nce.editor;


import nce.ColoringEditorPlugin;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.IStatusField;

public class ColoringEditor extends TextEditor {

	public ColoringEditor() {
		super();
	}
	
    @Override
	protected void doSetInput(IEditorInput input) throws CoreException {
		setDocumentProvider(createDocumentProvider(input));
		super.doSetInput(input);
	}

	private IDocumentProvider createDocumentProvider(IEditorInput input) {
		
//		if(input instanceof IFileEditorInput){
//			return new ColoringEditorDocumentProvider();
//		} else if(input instanceof IURIEditorInput){
//			return new ColoringEditorTextFileDocumentProvider();
//		}
		
		return new ColoringEditorDocumentProvider();
	}

	@Override
	protected void initializeEditor() {
    	
		super.initializeEditor();
		
		IPreferenceStore preferenceStore = new ChainedPreferenceStore(
				new IPreferenceStore[] { 
						ColoringEditorPlugin.getDefault().getPreferenceStore(), 
						getPreferenceStore() 
				});
 
		SourceViewerConfiguration configuration = 
				new ColoringEditorConfiguration(
						preferenceStore,
						ColoringEditorPlugin.getDefault().getColorManager()
				);
 
		setPreferenceStore(preferenceStore);
		setSourceViewerConfiguration(configuration);			
	}

	@Override
	protected ISourceViewer createSourceViewer(Composite parent, IVerticalRuler ruler, int styles) {
		return new ColoringEditorSourceViewer(parent, ruler, styles);
	}

	protected void updateStatusField(String category) {
		
        if (category == null) return;
        
        if (category.equals(ColoringEditorPlugin.STATUS_CATEGORY_MODE)) {
        	
            IStatusField field = getStatusField(category);
            if (field != null) {
            	
                field.setImage(ColoringEditorPlugin.getImage("cube"));
                String modeName = ((ColoringEditorDocumentProvider) getDocumentProvider()).getModeDisplayName();
                field.setText(createDisplayName(isNullOrEmpty(modeName) ? "No mode" : modeName));
                
                return;
            }
        }
        
        super.updateStatusField(category);
    }

	@Override
	protected void handlePreferenceStoreChanged(PropertyChangeEvent event) {
		//ColoringEditorPreferencePage.dumpColorsToLog();
		((ColoringEditorConfiguration) getSourceViewerConfiguration()).adaptToPreferenceChange(event);
		//getSourceViewer().invalidateTextPresentation(); // does not trigger a redraw of the complete text in JunoSDK
		super.handlePreferenceStoreChanged(event);
	}
	
    private String createDisplayName(String modeName) {
    	return Character.toUpperCase(modeName.charAt(0)) + modeName.substring(1);
    }
    
	private boolean isNullOrEmpty(String str) {
		return str == null || str.isEmpty();
	}

}
